<?php
namespace Vtb\VtbPay\Controller\Payment;

use Exception,
    Magento\Framework\App\Action\Action,
    Magento\Framework\App\Action\Context,
    Magento\Framework\App\Config\ScopeConfigInterface,
    Magento\Sales\Model\Order\Payment\Transaction\BuilderInterface as TransactionBuilder,
    Magento\Customer\Model\Session as CustomerSession,
    Magento\Sales\Api\OrderRepositoryInterface,
    Magento\Framework\Exception\LocalizedException,
    Magento\Sales\Model\Order\Payment\Transaction,
    Magento\Sales\Model\Order,
    Vtb\VtbPay\Model\VtbPay,
    Vtb\VtbPay\Helper\VtbPayLogger;


/**
 * Class HandleResponse
 *
 * This class is used to handle the response from the VTB payment system.
 * After a customer makes a payment, the payment system redirects the customer to this controller.
 * This controller fetches the order based on the request or the checkout session and checks the payment status.
 * If the payment is successful, it updates the order status, creates a transaction and redirects the customer to the order view page.
 * If the payment fails, it redirects the customer back to the checkout page and displays an error message.
 *
 * @package Vtb\VtbPay\Controller\Payment
 */
class HandleResponse extends Action
{

    /**
     * @var ScopeConfigInterface
     */
    protected $scopeConfig;
    
    /**
     * @var TransactionBuilder
     */
    protected $transactionBuilder;
    
    /**
     * @var OrderRepositoryInterface
     */
    protected $orderRepository;
    
    /**
     * @var CustomerSession
     */
    protected $customerSession;

    /**
     * @var VtbPayLogger
     */
    protected $vtbPayLogger;
    

    /**
     * HandleResponse constructor.
     *
     * @param Context $context
     * @param ScopeConfigInterface $scopeConfig
     * @param TransactionBuilder $transactionBuilder
     * @param OrderRepositoryInterface $orderRepository
     * @param CustomerSession $customerSession
     * @param VtbPayLogger $vtbPayLogger
     */
    public function __construct(
        Context $context,
        ScopeConfigInterface $scopeConfig,
        TransactionBuilder $transactionBuilder,
        OrderRepositoryInterface $orderRepository,
        CustomerSession $customerSession,
        VtbPayLogger $vtbPayLogger
    ) {
        parent::__construct($context);
        
        $this->scopeConfig = $scopeConfig;
        $this->transactionBuilder = $transactionBuilder;
        $this->orderRepository = $orderRepository;
        $this->customerSession = $customerSession;
        $this->vtbPayLogger = $vtbPayLogger;
    }


    /**
     * Main execution method.
     *
     * This method is responsible for handling the response from the VTB payment system.
     * It fetches the order, checks the payment status, updates the order status, creates a transaction and redirects
     * the customer to the correct page based on the outcome of the payment.
     *
     * @return \Magento\Framework\Controller\Result\Redirect|\Magento\Framework\View\Result\Page
     */
    public function execute()
    {
        // Get order ID from request params, or from session
        $orderId = $this->getRequest()->getParam('orderId');

        try {
            if (empty($orderId)) {
                throw new LocalizedException(__('Order ID not found.'));
            }
            
            // Load the order
            $order = $this->orderRepository->get($orderId);
            if (!$order->getId()) {
                throw new LocalizedException(__('This order does not exist.'));
            }
            
            /**
             * @var VtbPay
             */
            $payment = $order->getPayment();
            $vtbpay = $payment->getMethodInstance();
            if ($vtbpay->getCode() !== VtbPay::PAYMENT_METHOD_VTBPAY_CODE) {
                throw new LocalizedException(__('Unknown payment method.'));
            }
            
            $paymentStatus = $vtbpay->getPaymentStatus($order);
            if ($paymentStatus !== 'PAID') {
              throw new LocalizedException(__('Payment status is not PAID.'));
            }
              
            $endStatus = $this->scopeConfig->getValue(
                VtbPay::PAYMENT_METHOD_VTBPAY_XML_PATH . 'payment_vtbpay_transaction_end_status', 
                ScopeConfigInterface::SCOPE_TYPE_DEFAULT
            );
            // Установка оплаченной суммы и статуса заказа
            $order->setTotalPaid($order->getGrandTotal());
            $order->setBaseTotalPaid($order->getBaseGrandTotal());
            $order->setState(Order::STATE_COMPLETE)
                  ->setStatus($endStatus);

            $this->createTransaction($order, $payment);

            $this->orderRepository->save($order);
            
            $this->messageManager->addSuccessMessage(__('Payment was successful.') . ' Order # ' . $order->getIncrementId());
            
            // Redirect to the order view page
            if ($this->customerSession->isLoggedIn()) {
                $pathRedirect = 'sales/order/view/order_id/' . $order->getId();
            } 
            else {
                $pathRedirect = 'sales/guest/form/';
            }
            
        } catch (Exception | LocalizedException $e) {
            
            $this->vtbPayLogger->log($this, 'error', sprintf(
                __FUNCTION__ . ' > VtbPay Exception: %s; Order id: %s', 
                $e->getMessage(), 
                $orderId ?: ''
            ), [
                'code_exception' => $e->getCode(),
                'file_exception' => $e->getFile(),
                'line_exception' => $e->getLine(),
            ]);

            $this->messageManager->addErrorMessage(__('Payment failed.') . ' ' . $e->getMessage());

            $pathRedirect = 'checkout/cart';
        }
        
        $resultRedirect = $this->resultRedirectFactory->create();
        $resultRedirect->setPath($pathRedirect);

        return $resultRedirect;
    }


    /**
     * Creates a capture type transaction for the given order and payment.
     *
     * @param Order $order The order for which the transaction is to be created.
     * @param \Magento\Payment\Model\InfoInterface $payment The payment info instance.
     * 
     * @return void
     */
    private function createTransaction($order, $payment)
    {
        $transaction = $this->transactionBuilder
            ->setPayment($payment)
            ->setOrder($order)
            ->setTransactionId($order->getIncrementId())
            ->setAdditionalInformation(
                [Transaction::RAW_DETAILS => (array)$payment->getAdditionalInformation()]
            )
            ->setFailSafe(true)
            ->build(Transaction::TYPE_CAPTURE);
            
        $payment->addTransactionCommentsToOrder(
            $transaction,
            __('The payment transaction has been captured.')
        );
    }
}
