<?php
namespace Vtb\VtbPay\Model;

use Exception,
    Magento\Payment\Model\Method\AbstractMethod,
    Magento\Sales\Model\Order\Payment\Transaction\BuilderInterface,
    Magento\Sales\Model\Order,
    Magento\Framework\App\Config\ScopeConfigInterface,
    Magento\Quote\Api\Data\CartInterface,
    Magento\Framework\Exception\LocalizedException,
    Magento\Framework\Model\Context,
    Magento\Framework\Registry,
    Magento\Framework\Api\ExtensionAttributesFactory,
    Magento\Framework\Api\AttributeValueFactory,
    Magento\Payment\Helper\Data,
    Magento\Payment\Model\Method\Logger,
    Magento\Checkout\Model\Session,
    Magento\Sales\Model\OrderFactory,
    Magento\Framework\UrlInterface,
    Vtb\VtbPay\Gateway\Http\Client\VtbApi,
    Vtb\VtbPay\Helper\VtbPayLogger;


/**
 * VtbPay Payment Method
 * 
 * This class extends Magento's AbstractMethod to provide VtbPay as a payment method.
 * It defines several properties and methods required by the Magento payment method system, including code, form block type,
 * and info block type.
 * It also interacts with the VtbApi to get and provide order information to and from the VtbPay system.
 *
 * @package Vtb\VtbPay\Model
 */
class VtbPay extends AbstractMethod
{
    /**
     * These boolean variables define the capabilities of the VtbPay payment method.
     * It declares the method as a gateway method, capable of order, authorize, capture, and use for internal transactions
     * and checkout, but not capable of refunds or multi-shipping transactions.
     */
    protected $_isGateway = true;
    protected $_isOffline = false;
    protected $_canOrder = true;
    protected $_canAuthorize = true;
    protected $_canCapture = true;
    protected $_canRefund = false;
    protected $_canUseInternal = true;
    protected $_canUseCheckout = true;
    protected $_canUseForMultishipping = false;
    protected $_isInitializeNeeded = true;

    /**
     * @var string Payment method code and XML path for VtbPay configuration data.
     */
    public const PAYMENT_METHOD_VTBPAY_CODE = 'vtb_vtbpay';
    public const PAYMENT_METHOD_VTBPAY_XML_PATH = 'payment/vtb_vtbpay/';

    /**
     * @var string Payment method code for this module.
     */
    protected $_code = self::PAYMENT_METHOD_VTBPAY_CODE;

    /**
     * @var string Path to the template for the VtbPay payment information block.
     */
    protected $_infoBlockType = \Magento\Payment\Block\Info\Instructions::class;

    /**
     * @var \Vtb\VtbPay\Model\VtbApi Reference to the VtbApi object that performs interactions with the VtbPay system.
     */
    protected $vtbApi;

    /**
     * @var Session
     */
    protected $checkoutSession;

    /**
     * @var OrderFactory
     */
    protected $orderFactory;

    /**
     * @var BuilderInterface
     */
    protected $transactionBuilder;

    /**
     * @var UrlInterface
     */
    protected $_urlBuilder;

    /**
     * @var VtbPayLogger
     */
    protected $vtbPayLogger;


    /**
     * The constructor sets up the necessary dependencies for the class.
     *
     * @param Context $context Application context object
     * @param Registry $registry Registry object
     * @param ExtensionAttributesFactory $extensionFactory Extension attributes factory object
     * @param AttributeValueFactory $customAttributeFactory Custom attribute factory object
     * @param Data $paymentData Payment helper object
     * @param ScopeConfigInterface $scopeConfig Scope configuration object
     * @param Logger $logger Logger object
     * @param Session $checkoutSession Checkout session object
     * @param OrderFactory $orderFactory Order factory object
     * @param BuilderInterface $transactionBuilder Transaction builder object
     * @param UrlInterface $urlBuilder URL builder object
     * @param VtbPayLogger $vtbPayLogger Custom VtbPay logger object
     */
    public function __construct(
        Context $context,
        Registry $registry,
        ExtensionAttributesFactory $extensionFactory,
        AttributeValueFactory $customAttributeFactory,
        Data $paymentData,
        ScopeConfigInterface $scopeConfig,
        Logger $logger,
        Session $checkoutSession,
        OrderFactory $orderFactory,
        BuilderInterface $transactionBuilder,
        UrlInterface $urlBuilder,
        VtbPayLogger $vtbPayLogger
    ) {
        $this->_checkoutSession = $checkoutSession;
        $this->_orderFactory = $orderFactory;
        $this->_transactionBuilder = $transactionBuilder;
        $this->_urlBuilder = $urlBuilder;
        $this->vtbPayLogger = $vtbPayLogger;

        $this->setVtbApi($scopeConfig);

        parent::__construct(
            $context,
            $registry,
            $extensionFactory,
            $customAttributeFactory,
            $paymentData,
            $scopeConfig,
            $logger
        );
    }


    /**
     * setVtbApi() initializes the VtbApi object with configuration data.
     *
     * @param ScopeConfigInterface $scopeConfig Scope configuration object
     */
    protected function setVtbApi($scopeConfig)
    {
        $arFields = [
            'client_id',
            'client_secret',
            'test_mode',
            'merchant_authorization'
        ];
        
        $arFieldsData = [];
        
        foreach ($arFields as $arField) {
            $arFieldsData[$arField] = trim($scopeConfig->getValue(
                self::PAYMENT_METHOD_VTBPAY_XML_PATH . 'payment_vtbpay_' . $arField, 
                ScopeConfigInterface::SCOPE_TYPE_DEFAULT
            ));
        }
        
        $this->vtbApi = new VtbApi(
            $arFieldsData['client_id'], 
            $arFieldsData['client_secret'], 
            $arFieldsData['test_mode'], 
            $arFieldsData['merchant_authorization']
        );
    }


    /**
     * initialize() method sets the initial order status and state.
     *
     * @param string $paymentAction Payment action string
     * @param \Magento\Framework\DataObject $stateObject State object
     * @return PaymentMethod Returns this class instance.
     */
    public function initialize($paymentAction, $stateObject): PaymentMethod
    {
        $stateObject->setStatus('new');
        $stateObject->setState(Order::STATE_PENDING_PAYMENT);
        $stateObject->setIsNotified(false);

        return $this;
    }


    /**
     * getCheckoutRedirect() method requests a checkout URL from the VtbApi
     * and returns it, throwing an exception if no URL is received.
     *
     * @param Order $order Order object
     * @return string Returns checkout URL string.
     * @throws LocalizedException Throws exception when failed to get payment URL.
     */
    public function getCheckoutRedirect($order)
    {
        $orderId = $order->getId();

        $response = $this->vtbApi->getOrderLink(
            $orderId, 
            $order->getCustomerEmail(), 
            time(), 
            $order->getGrandTotal(), 
            $this->getReturnUrl()
        );

        $this->vtbPayLogger->log($this, 'debug', __FUNCTION__ . ' > getOrderLink', [
            'arguments' => [
                'orderId' => $orderId, 
                'email' => $order->getCustomerEmail(), 
                'time' => time(), 
                'total' => $order->getGrandTotal(), 
                'returnUrl' => $this->getReturnUrl()
            ],
            'response' => $response
        ]);

        if (
            !isset($response['object']['payUrl']) || 
            empty($payUrl = $response['object']['payUrl'])
        ) {
            throw new LocalizedException(__('Failed to get payment URL.'));
        }

        return $payUrl;
    }


    /**
     * getReturnUrl() method returns the URL that the VtbPay gateway should redirect
     * the customer to after they complete their payment.
     *
     * @return string Returns URL string.
     */
    protected function getReturnUrl()
    {
        // You might need to adjust the exact URL depending on your module structure.
        return $this->_urlBuilder->getUrl('vtbpay/payment/handleResponse', ['_secure' => true]);
    }


    /**
     * getPaymentStatus() method requests the payment status from the VtbApi
     * for a specific order and returns it, throwing an exception if no status is received.
     *
     * @param Order $order Order object
     * @return string Returns payment status string.
     * @throws LocalizedException Throws exception when no information about payment status.
     */
    public function getPaymentStatus($order)
    {
        $orderId = $order->getId();
        
        $response = $this->vtbApi->getOrderInfo($orderId);
        
        $this->vtbPayLogger->log($this, 'debug', __FUNCTION__ . ' > getOrderInfo', [
            'arguments' => [
                'orderId' => $orderId
            ],
            'response' => $response
        ]);

        if (
            !isset($response['object']['status']['value']) ||
            empty($status = $response['object']['status']['value'])
        ) {
            throw new LocalizedException(__('No information about payment status.'));
        }

        return $status;
    }


    /**
     * isAvailable() method checks if the payment method is available for use
     * based on whether or not it is active in the configuration.
     *
     * @param CartInterface|null $quote Cart quote object
     * @return bool Returns true if the payment method is active, otherwise false.
     */
    public function isAvailable(CartInterface $quote = null): bool
    {
        if (!$this->isActive($quote ? $quote->getStoreId() : null)) {
            return false;
        }
        return true;
    }


    /**
     * getInstructions() method fetches the instruction text configured for the payment method.
     *
     * @return string Returns instruction text string.
     */
    public function getInstructions()
    {
        $instructions = $this->getConfigData('instructions');
        return !empty($instructions) ? trim($instructions) : '';
    }
}